#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.add_kqi_temporal_frequency_input import AddKqiTemporalFrequencyInput


# fmt: off
QUERY: List[str] = ["""
mutation addKqiTemporalFrequency($input: AddKqiTemporalFrequencyInput!) {
  addKqiTemporalFrequency(input: $input) {
    id
    name
  }
}
"""
]


class addKqiTemporalFrequency:
    @dataclass(frozen=True)
    class addKqiTemporalFrequencyData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class KqiTemporalFrequency(DataClassJsonMixin):
            id: str
            name: str

        addKqiTemporalFrequency: KqiTemporalFrequency

    # fmt: off
    @classmethod
    def execute(cls, client: Client, input: AddKqiTemporalFrequencyInput) -> addKqiTemporalFrequencyData.KqiTemporalFrequency:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.addKqiTemporalFrequencyData.from_dict(response_text)
        return res.addKqiTemporalFrequency

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, input: AddKqiTemporalFrequencyInput) -> addKqiTemporalFrequencyData.KqiTemporalFrequency:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.addKqiTemporalFrequencyData.from_dict(response_text)
        return res.addKqiTemporalFrequency
