#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.survey_question_type import SurveyQuestionType

# fmt: off
QUERY: List[str] = ["""
fragment SurveyQuestionFragment on SurveyQuestion {
  formName
  questionFormat
  questionText
  boolData
  emailData
  latitude
  longitude
  phoneData
  textData
  floatData
  intData
  dateData
}

"""]

@dataclass(frozen=True)
class SurveyQuestionFragment(DataClassJsonMixin):
    formName: Optional[str]
    questionFormat: Optional[SurveyQuestionType] = _field(metadata=enum_field_metadata(SurveyQuestionType))
    questionText: str
    boolData: Optional[bool]
    emailData: Optional[str]
    latitude: Optional[Number]
    longitude: Optional[Number]
    phoneData: Optional[str]
    textData: Optional[str]
    floatData: Optional[Number]
    intData: Optional[int]
    dateData: Optional[int]
